@@ -2,18 +2,22 @@
 #
 # Inline::Octave - 
 #
-# $Id: Octave.pm,v 1.5 2001/11/11 03:36:31 aadler Exp $
+# $Id: Octave.pm,v 1.8 2001/11/18 03:29:06 aadler Exp $
 
 package Inline::Octave;
 
 
-$VERSION = '0.10';
+$VERSION = '0.14';
 require Inline;
 @ISA = qw(Inline);
 use Carp;
 use IPC::Open2;
-use vars qw( $inline_object );
+use vars qw( $octave_object );
 
+# set values which should change to this,
+# if it doesn't change, we have an error.
+my $retcode_string= "[-101101.101101,-918273.6455,-178.9245867]";
+my $retcode_value=   [-101101.101101,-918273.6455,-178.9245867];
 
 sub register {
 #  print "REGISTERING\n";
@@ -27,11 +31,21 @@ sub register {
 
 
 sub build {
+   my $o = shift;
+   $o->_build (@_ );
+}
+
+sub _build {
 #  print "BUILDING\n";
    my $o = shift;
    my $code = $o->{API}{code};
 
-   {
+   # we don't really need to validate the code here
+   # since that gets done in load anyway
+   #
+   # Also, stopping the interpreter will make any
+   # functions defined in a previous section disappear
+   if (0) {
       $o->start_interpreter();
       print $o->interpret($code);
       my @def_funcs= $o->get_defined_functions();
@@ -53,7 +67,6 @@ sub load {
 #  print "LOADING\n";
    my $o = shift;
    $o->_validate();
-   $inline_object = $o;
    
    my $obj = $o->{API}{location};
    open OCTAVE_OBJ, "< $obj" or croak "Can't open $obj for output\n$!";
@@ -67,15 +80,14 @@ sub load {
          #TODO make this better - ie loop
          my $nargout=0;
             $nargout= 1 if $pat =~ /^\w+\s*=/;
-            $nargout= 1 if $pat =~ /^\[\s*\w+\s*\]\s*=/;
-            $nargout= 2 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*]\s*=/;
-            $nargout= 3 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*,\s*\w+\s*]\s*=/;
-            $nargout= 4 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*]\s*=/;
-            $nargout= 5 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*]\s*=/;
-            $nargout= 6 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*]\s*=/;
-            $nargout= 7 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*]\s*=/;
-            $nargout= 8 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*]\s*=/;
-            $nargout= 9 if $pat =~ /^\[\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*,\s*\w+\s*]\s*=/;
+
+         if ($pat =~ /^\[([\s\w,]+)\]\s*=/ ) {
+            my @fnpat = split /,/, $1;
+            foreach (@fnpat) {
+               $nargout++ if /^\s*\w+\s*$/;
+            }
+         }
+
          $nargouts{$fnam}=$nargout;
       }
       if (/^\s*##\s*Inline::Octave::(\w+)\s*\(nargout=(\d+)\)\s*=>\s*(\w*)/) {
@@ -86,16 +98,16 @@ sub load {
    close OCTAVE_OBJ;
 #  use Data::Dumper; print Dumper(\%nargouts);
    
-   {
-       $o->start_interpreter();
-       print $o->interpret($code);
-       my @def_funcs= $o->get_defined_functions();
-       foreach my $funname (@def_funcs) {
-          $o->bind_octave_function( $funname, $funname, $nargouts{$funname} );
-       }
-       @EXPORT= @def_funcs;
+   $o->start_interpreter();
+
+   print $o->interpret($code);
+   my @def_funcs= $o->get_defined_functions();
+   foreach my $funname (@def_funcs) {
+      next if defined $octave_object->{FUNCS}->{$funname};
+      $o->bind_octave_function( $funname, $funname, $nargouts{$funname} );
    }
-   croak "Unable to load Octave module $obj:\n$@" if $@;
+
+   return;
 }
 
 
@@ -103,23 +115,33 @@ sub validate
 {
 # print "VALIDATING\n";
   my $o = shift;
-  $o->_validate();
+  $o->_validate( @_ );
 }
 
 sub _validate
 {
   my $o = shift;
-  $o->{ILSM}->{INTERP} = "octave -qf "
-     unless exists $o->{ILSM}->{INTERP};
-  $o->{ILSM}->{MARKER} = "-9Ahv87uhBa8l_8Onq,zU9-"
-     unless exists $o->{ILSM}->{MARKER};
+
+  my $switches= "-qfH";
+  while (@_) {
+     my ($key, $value) = (shift, shift) ;
+     if ($key eq 'OCTAVE_BIN'){
+         $octave_object->{INTERP} = "$value $switches ";
+     } 
+#    print "$key--->$value\n";
+  }
+  $octave_object->{INTERP} = "$ENV{PERL_INLINE_OCTAVE_BIN} $switches "
+     if $ENV{PERL_INLINE_OCTAVE_BIN};
+  $octave_object->{INTERP} = "octave $switches "
+     unless exists $octave_object->{INTERP};
+  $octave_object->{MARKER} = "-9Ahv87uhBa8l_8Onq,zU9-"
+     unless exists $octave_object->{MARKER};
 }   
 
 sub info
 {
-  print "INFO\n";
-  my $o = shift;
-  # Place holder
+#  print "INFO\n";
+   my $o = shift;
 }
 
 
@@ -147,7 +169,7 @@ sub $perl_funname {
    # in the loop, but rather at the end of the function
    
    #input variables
-   my \$inargs;
+   my \$inargs=" ";
    my \@vin;
    for (my \$i=0; \$i < \@_; \$i++) {
       \$vin[\$i]= new Inline::Octave::Matrix( \$_[\$i] );
@@ -156,10 +178,10 @@ sub $perl_funname {
    chop(\$inargs); #remove last ,
 
    #output variables
-   my \$outargs;
+   my \$outargs=" ";
    my \@vout;
    for (my \$i=0; \$i < $nargout; \$i++) {
-      \$vout[\$i]= new Inline::Octave::Matrix( -101101.101101 ); #code
+      \$vout[\$i]= new Inline::Octave::Matrix( $retcode_string );
       \$outargs.= \$vout[\$i]->name.",";
    }
    chop(\$outargs); #remove last ,
@@ -168,7 +190,7 @@ sub $perl_funname {
 
    my \$call= "\$outargs $oct_funname(\$inargs);";
 #  print "--\$call--\\n";
-   my \$retval= \$Inline::Octave::inline_object->interpret( \$call );
+   my \$retval= Inline::Octave::interpret(0, \$call );
 #  print "--\$retval--\\n";
 
    # Get the correct size for each new variable
@@ -181,41 +203,83 @@ CODE
 #  print "--$code--\n";
    eval $code;
    croak "Problem binding $oct_funname to $perl_funname: $@" if $@;
+
+   $octave_object->{FUNCS}->{$oct_funname}= $perl_funname;
+   return;
 }   
 
 sub start_interpreter
 {
    my $o = shift;
 
+   # check if interpreter already alive
+   return if $octave_object->{OCTIN} and $octave_object->{OCTOUT};
+
    my $Oout; my $Oin;
+   my $pid;
    eval {
 # This works in perl 5.6
-#     open2( $Oout, $Oin , $o->{ILSM}->{INTERP} ); 
+#     open2( $Oout, $Oin , $octave_object->{INTERP} ); 
 # But we need to do this in 5.005     
-      open2( \*OOUT, \*OIN , $o->{ILSM}->{INTERP} );
+      $pid= open2( \*OOUT, \*OIN , $octave_object->{INTERP} );
       $Oout= \*OOUT; $Oin= \*OIN;
    };
    croak "Can't locate octave interpreter: $@\n" if $@ =~ /Open2/i;
 
-   $o->{ILSM}->{OCTIN} = $Oin;
-   $o->{ILSM}->{OCTOUT} = $Oout;
-}       
+#  $SIG{CHLD}= \&reap_interpreter;
+#  $SIG{PIPE}= \&reap_interpreter;
+
+   $octave_object->{octave_pid} = $pid;
+   $octave_object->{OCTIN} = $Oin;
+   $octave_object->{OCTOUT} = $Oout;
+
+   # some of this is necessary, some are the defaults
+   # but it never hurts to be cautious
+   my $startup_code= <<STARTUP_CODE;
+crash_dumps_octave_core=0;
+page_screen_output=0;
+silent_functions=1;
+do_fortran_indexing=1; 
+page_screen_output=0;
+page_output_immediately=1;
+STARTUP_CODE
+
+   $o->interpret( $startup_code ); # check return value?
+
+   return;
+}
+
+sub reap_interpreter
+{
+#  print "REAP_INTERPRETER\n";
+   my $o= $octave_object;
+   my $pid= $octave_object->{octave_pid};
+   return unless $pid;
+
+   waitpid $pid,0;
+   $octave_object->{OCTIN} = "";
+   $octave_object->{OCTOUT} = "";
+   $octave_object->{octave_pid} = "";
+   return;
+}   
 
 sub stop_interpreter
 {
    my $o = shift;
 
-   my $Oin= $o->{ILSM}->{OCTIN};
-   my $Oout= $o->{ILSM}->{OCTOUT};
+   my $Oin= $octave_object->{OCTIN};
+   my $Oout= $octave_object->{OCTOUT};
 
    return unless $Oin and $Oout;
 
-   print $Oin "exit\n";
+   print $Oin "\n\nexit\n";
    #<$Oin>; #clean up input - is this required?
    close $Oin;
    close $Oout;
-   $o->{ILSM}->{OCTIN} = "";
-   $o->{ILSM}->{OCTOUT} = "";
+   $octave_object->{OCTIN} = "";
+   $octave_object->{OCTOUT} = "";
+   $octave_object->{octave_pid} = "";
+   return;
 }   
 
 # send a string to octave and get the result
@@ -223,14 +287,16 @@ sub interpret
 {
    my $o = shift;
    my $cmd= shift;
-   my $marker= $o->{ILSM}->{MARKER};
+   my $marker= $octave_object->{MARKER};
 
-   my $Oin= $o->{ILSM}->{OCTIN};
-   my $Oout= $o->{ILSM}->{OCTOUT};
+   my $Oin= $octave_object->{OCTIN};
+   my $Oout= $octave_object->{OCTOUT};
 
    croak "octave interpreter not alive"  unless $Oin and $Oout;
 
-   print $Oin "$cmd\ndisp('$marker');fflush(stdout);\n";
+#  print "INTERP: $cmd\n";
+
+   print $Oin "\n\n$cmd\ndisp('$marker');fflush(stdout);\n";
 
    my $input;
    my $marker_len= length( $marker )+1;
@@ -239,6 +305,10 @@ sub interpret
       $input.= $line;
       last if substr( $input, -$marker_len, -1) eq $marker;
    }   
+
+   # we need to leave octave blocked doing something,
+   # otherwise it can't handle a CTRL-C
+   print $Oin "\n\nfread(stdin,1);\n";
    return substr($input,0,-$marker_len);
 }   
 
@@ -256,13 +326,14 @@ sub get_defined_functions
 }       
 
 END {
-   $inline_object->stop_interpreter() if $inline_object;
+#  print "ENDING\n";
+   Inline::Octave::stop_interpreter() if $octave_object;
 }
 
 package Inline::Octave::Matrix;
 use Carp;
 
-$varcounter= 100001;
+$varcounter= 10000001;
 # called as
 # new IOM( [1,2,3] ) -> ColumnVector
 # new IOM( [[1,2],[2,3],[3,4]] ) -> Matrix
@@ -325,7 +396,7 @@ sub new
              pack( "d".($rows*$cols) , @vals );
    }
 
-   $Inline::Octave::inline_object->interpret( $code );
+   Inline::Octave::interpret(0, $code );
    $self->store_size();
 
    return $self;
@@ -336,8 +407,8 @@ sub store_size
    my $self = shift;
    my $varname= $self->name;
    my $code = "disp([size($varname), is_complex($varname)] )";
-   my $size=  $Inline::Octave::inline_object->interpret( $code );
-   croak "Problem constructing Matrix" unless $size =~ /^ *(\d+) *(\d+) *[01]/;
+   my $size=  Inline::Octave::interpret(0, $code );
+   croak "Problem constructing Matrix" unless $size =~ /^ +(\d+) +(\d+) +[01]/;
    $self->{rows}= $1;
    $self->{cols}= $2;
    $self->{complex}= $3;
@@ -349,7 +420,7 @@ sub as_list
    my $varname= $self->name;
    croak "Can't handle complex" if $self->{complex};
    my $code = "fwrite(stdout, $varname,'double');";
-   my $retval= $Inline::Octave::inline_object->interpret( $code );
+   my $retval= Inline::Octave::interpret(0, $code );
    my $size= $self->{cols} * $self->{rows};
    my @list= unpack "d$size", $retval;
    return @list;
@@ -361,7 +432,7 @@ sub as_matrix
    my $varname= $self->name;
    croak "Can't handle complex" if $self->{complex};
    my $code = "fwrite(stdout, $varname','double');"; # use transpose
-   my $retval= $Inline::Octave::inline_object->interpret( $code );
+   my $retval= Inline::Octave::interpret(0, $code );
    my $size= $self->{cols} * $self->{rows};
    my @list= unpack "d$size", $retval;
    my @m;
@@ -382,7 +453,7 @@ sub as_scalar
            $self->{cols}."x".$self->{rows}
            unless $self->{cols} == 1 && $self->{rows} == 1;
    my $code = "fwrite(stdout, $varname,'double');";
-   my $retval= $Inline::Octave::inline_object->interpret( $code );
+   my $retval= Inline::Octave::interpret(0, $code );
    my @list= unpack "d1", $retval;
    return $list[0];
 }   
@@ -393,7 +464,7 @@ sub DESTROY
    my $self = shift;
    my $varname= $self->name;
    my $code = "clear $varname;";
-   $Inline::Octave::inline_object->interpret( $code );
+   Inline::Octave::interpret(0, $code );
 }   
 
 sub disp
@@ -401,7 +472,7 @@ sub disp
    my $self = shift;
    my $varname= $self->name;
    my $code = "disp( $varname );";
-   return $Inline::Octave::inline_object->interpret( $code );
+   return Inline::Octave::interpret(0, $code );
 }   
 
 sub name
@@ -410,12 +481,91 @@ sub name
    return $self->{varname};
 }   
 
+
+#
+# Define arithmetic on IOMs 
+#
+use overload 
+    '+' => sub { oct_matrix_arithmetic( @_, '+',  ) },
+    '-' => sub { oct_matrix_arithmetic( @_, '-',  ) },
+    '*' => sub { oct_matrix_arithmetic( @_, '.*', ) },
+    '/' => sub { oct_matrix_arithmetic( @_, './', ) },
+    'x' => sub { oct_matrix_arithmetic( @_, '*',  ) };
+
+sub oct_matrix_arithmetic
+{
+   my $a= new Inline::Octave::Matrix( shift );
+   my $b= new Inline::Octave::Matrix( shift );
+   ($b,$a)= ($a,$b) if shift;
+   my $op= shift;
+
+   my $v= new Inline::Octave::Matrix( $retcode_value );
+
+   my $code= $v->name."=". $a->name ." $op ". $b->name .';';
+   run_math_code( $code, $v);
+   return $v;
+}   
+
+sub transpose
+{
+   my $a= new Inline::Octave::Matrix( shift );
+   my $v= new Inline::Octave::Matrix( $retcode_value );
+   my $code= $v->name."=". $a->name.".';";
+   run_math_code( $code, $v);
+   return $v;
+}  
+
+# run_math_code ( $code, $val )
+# dies on error, no return
+sub run_math_code
+{
+   my $code= shift;
+   my $v   = shift;
+   my $vname= $v->name;
+
+   $runcode= "$code  disp (size($vname)==[3,1] && ".
+             " all($vname==$retcode_string') );\n";
+   my $retval= Inline::Octave::interpret(0, $runcode );
+
+   if ($retval == 0) {
+      $v->store_size();
+   } else {
+      croak "Error performing operation $code";
+   }
+}      
+
 1;
 
 
 __END__
 
+TODO LIST:
+
+   1. Add import for functions
+   2. control matrix size inputs
+   3. add destructor for Octave::Matrix
+       - done
+   4. control waiting in the interpret loop
+       - seems ok, except sysread reads small buffers
+   5. support for complex variables
+   6. octave gets wierd when you CTRL-C out of a 
+       running program
+       - seems ok
+   7. Use parse-recdecent to parse octave code
+   8. Come up with an OO way to avoid
+       Inline::Octave::interpret(0, $code );
+
 $Log: Octave.pm,v $
+Revision 1.8  2001/11/18 03:29:06  aadler
+bug in fread fix - add \n
+
+Revision 1.7  2001/11/18 03:22:42  aadler
+multisections now ok, cleaned up singleton object,
+octave no longer freaks out on ctrl-c
+
+Revision 1.6  2001/11/17 02:15:21  aadler
+changed docs, new options for Makefile.PL
+
 Revision 1.5  2001/11/11 03:36:31  aadler
 mod to work with octave-2.0 as well as 2.1
 
@@ -446,6 +596,9 @@ Inline::Octave - Inline octave code into your perl
 
    oct_plot( [0..4], [3,2,1,2,3] );
    sleep(2);
+
+   my $d= (2*$c) x $c->transpose;
+   print $d->disp;
    
    
    __DATA__
@@ -461,25 +614,17 @@ Inline::Octave - Inline octave code into your perl
    ## Inline::Octave::oct_sum (nargout=1)  => sum
    ## Inline::Octave::oct_plot (nargout=0)  => plot
 
+=head1   WARNING
+
+THIS IS ALPHA SOFTWARE. It is incomplete and possibly unreliable.  It is
+also possible that some elements of the interface (API) will change in
+future releases. 
 
 =head1 DESCRIPTION
 
 Inline::Octave gives you the power of the octave programming language from
 within your Perl programs.
 
-You need to install the Inline module from CPAN. This provides
-the infrastructure to support all the Inline::* modules.
- 
-Then install Octave.pm in an Inline directory in any path in your @INC.
- 
-The easiest is to create a ./Inline directory in your
-working directory, and put Octave.pm in that.
-after that the example code should run.
-
-Note, there is currently no proper Makefile.PL install facility
-for Inline::Octave - this reflects it's maturity.
-                   
-It should work with stock octave - but I'd like to get feedback on this.
 Basically, I create an octave process with controlled stdin and stdout.
 Commands send by stdin. Data is send by stdin and read with
 fread(stdin, [dimx dimy], "double"), and read similarly.
@@ -490,7 +635,38 @@ command to octave.
                    
 I initially tried to bind the C++ and liboctave to perl, but
 it started to get really hard - so I took this route.
+I'm planning to get back to that eventually ...
 
+=head1 INSTALLATION
+
+You need to install the Inline module from CPAN. This provides
+the infrastructure to support all the Inline::* modules.
+
+Then:
+   
+   perl Makefile.PL
+   make 
+   make test
+   make install
+
+This will search for an octave interpreter and give you
+the choice of giving the path to GNU Octave.
+
+If you don't want this interactivity, then specify
+
+   perl Makefile.PL OCTAVE=/path/to/octave
+      or
+   perl Makefile.PL OCTAVE='/path/to/octave -my -special -switches'
+
+The path to the octave interpreter can be set in the following
+ways:
+
+   - set OCTAVE_BIN option in the use line
+
+      use Inline Octave => DATA => OCTAVE_BIN => /path/to/octave
+
+   - set the PERL_INLINE_OCTAVE_BIN environment variable
+ 
 =head1 Why would I use Inline::Octave
 
 If you can't figure out a reason, don't!
@@ -498,6 +674,11 @@ If you can't figure out a reason, don't!
 I use it to grind through long logfiles (using perl),
 and then calculate mathematical results (using octave).
 
+Why not use PDL?
+
+1) Because there's lots of existing code in Octave/Matlab.
+2) Because there's functionality in Octave that's not in PDL.
+3) Because there's more than one way to do it.
 
 =head1 Using Inline::Octave
 
@@ -603,6 +784,23 @@ Values can be created explicitly, using the syntax:
 or values will be automatically created by 
 calling octave functions.
 
+=head1 Operations on Inline::Octave::Matrix -es
+
+Many math operations have been overloaded to
+work directly on Inline::Octave::Matrix values;
+
+For example, given $var above, we can calculate:
+
+   $v1= ( $var x $var->transpose );
+   $v2=  2*$var + 1
+   $v3=  $var x [ [1],[2] ];
+
+The relation between Perl and Octave operators is:   
+      '+' => '+',
+      '-' => '-',
+      '*' => '.*',
+      '/' => './',
+      'x' => '*',
 
 =head1 PERFORMANCE
 
@@ -631,14 +829,3 @@ redistributed and/or modified under the same terms as Perl itself.
 
 
 
-=head1 TODO List
-
-   1. Add import for functions
-   2. control matrix size inputs
-   3. add destructor for Octave::Matrix
-       - done
-   4. control waiting in the interpret loop
-       - seems ok, except sysread reads small buffers
-   5. support for complex variables
-
-
